//=================================================================================================
/*!
//  \file blaze/config/SMP.h
//  \brief Configuration of the shared-memory parallelization
//
//  Copyright (C) 2012-2020 Klaus Iglberger - All Rights Reserved
//
//  This file is part of the Blaze library. You can redistribute it and/or modify it under
//  the terms of the New (Revised) BSD License. Redistribution and use in source and binary
//  forms, with or without modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice, this list
//     of conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//  3. Neither the names of the Blaze development group nor the names of its contributors
//     may be used to endorse or promote products derived from this software without specific
//     prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
*/
//=================================================================================================


//*************************************************************************************************
/*!\brief Compilation switch for the (de-)activation of the shared-memory parallelization
// \ingroup config
//
// This compilation switch enables/disables the shared-memory parallelization. In case the switch
// is set to 1 (i.e. in case the shared-memory parallelization is enabled), the Blaze library is
// allowed to execute operations in parallel. In case the switch is set to 0 (i.e. parallelization
// is disabled), the Blaze library is restricted from executing operations in parallel.
//
// Possible settings for the shared-memory parallelization switch:
//  - Deactivated: \b 0
//  - Activated  : \b 1 (default)
//
// \note It is possible to (de-)activate the shared-memory parallelization via command line or
// by defining this symbol manually before including any Blaze header file:

   \code
   g++ ... -DBLAZE_USE_SHARED_MEMORY_PARALLELIZATION=1 ...
   \endcode

   \code
   #define BLAZE_USE_SHARED_MEMORY_PARALLELIZATION 1
   #include <blaze/Blaze.h>
   \endcode
*/
#ifndef BLAZE_USE_SHARED_MEMORY_PARALLELIZATION
#define BLAZE_USE_SHARED_MEMORY_PARALLELIZATION 1
#endif
//*************************************************************************************************
